;;;; File: gopher.lsp
;;;; SCCS Version: %W%
;;;; Contains: Graphical Operations Processor Handler of Entity Relations.
;;;; Author: Richard Kirby (rbk)
;;;; Created: Fri Apr 20 10:35:06 1990
;;;; Updated: Fri Mar 24 23:12:39 1995 by Jeff Dalton
;;;; Release Version: %Y%
;;;; Copyright: (c) 1992, AIAI, University of Edinburgh
;;;; This material may be reproduced by or for the U.S. Government pursuant
;;;; to the copyright license under the clause at DFARS 252.227-7032
;;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

(in-package 'gopher :nicknames '(gop))

(use-package :datastructures)

(use-package :util)

;;; Many of these functions were not used, and some the unused ones
;;; were wrong.  (Be very careful when comparing values from the TPN.)
;;; Since their presence in this file was somewhat misleading, the unused
;;; functions have been deleted and their names commented out of the export
;;; list.  [JWD 27 Mar 93]

(export '(gop-precedes
	  ; gop-finishes-before
	  gop-follows
	  ; gop-finishes-after
	  ; gop-parallel
	  ; gop-precedes-list
	  ; gop-finishes-before-list
	  ; gop-follows-list
	  ; gop-finishes-after-list
	  ; gop-parallel-list
	  ; gop-last-incoming
	  ; gop-early
	  ; gop-late
	  ))

;;;; The relationship functions.

;;; These take two node ends, and return t or nil depending on whether
;;; the relationship holds or not.

;;; /\/: Of course (gop-follows a b) could be defined as (gop-precedes b a)

(defun gop-precedes (query-tag focus-tag)
  "Does the latest time of query come before the earliest time of focus?"
  (let ((query (get-node-end query-tag))
	(focus (get-node-end focus-tag)))
    (or (<-or-bust (tpoint-max (ne-time-point query))
		   (tpoint-min (ne-time-point focus)))
	(and ;(not (eq query focus))
	     (is-linked-before-p query focus)))))


(defun gop-follows (query-tag focus-tag)
  "Does the latest time of focus come before the earliest time of query?"
  (let ((query (get-node-end query-tag))
	(focus (get-node-end focus-tag)))
    (or (<-or-bust (tpoint-max (ne-time-point focus))
		   (tpoint-min (ne-time-point query)))
	(and ;(not (eq query focus))
	     (is-linked-after-p query focus)))))


;;;; Net traversal functions

;;; GOP marks are used for preventing excessive work.

(defvar *gop-marks* (make-array 500 :element-type 'fixnum :initial-element 0))
(defvar *gop-mark* 0)

(defun get-new-gop-mark ()
  (when (> (node-end-count) (length *gop-marks*))
    (setq *gop-marks*
	  (make-array (round (* 3/2 (node-end-count)))
		      :element-type 'fixnum :initial-element 0)))
  (incf *gop-mark*)
  (when (= *gop-mark* most-positive-fixnum)
    (cerror "Will continue into bignums" "Cor, you've done a lot."))
  *gop-mark*)

;;; /\/: Avoids return-from in the local function, because it loses in AKCL.

(defun is-linked-before-p (query focus)
  ;; N.B. returns false if query and focus are the same.
  (declare (optimize (speed 3) (safety 0)))
  (get-new-gop-mark)
  (label walk ((consider focus))
    (let ((visited *gop-mark*)
	  (marks *gop-marks*))
      (declare (type fixnum visited)
	       (type (simple-array fixnum (*)) marks))
      (macrolet ((mark (ne) `(aref marks (ne-index ,ne))))
	(dolist (c (ne-pre-ends consider) nil)
	  (cond ((eq c query)
		 (return t))
		((not (= visited (mark c)))
		 (setf (mark c) visited)
		 (when (walk c)
		   (return t)))))))))

(defun is-linked-after-p (query focus)
  ;; N.B. returns false if query and focus are the same.
  (declare (optimize (speed 3) (safety 0)))
  (get-new-gop-mark)
  (label walk ((consider focus))
    (let ((visited *gop-mark*)
	  (marks *gop-marks*))
      (declare (type fixnum visited)
	       (type (simple-array fixnum (*)) marks))
      (macrolet ((mark (ne) `(aref marks (ne-index ,ne))))
	(dolist (c (ne-post-ends consider) nil)
	  (cond ((eq c query)
		 (return t))
		((not (= visited (mark c)))
		 (setf (mark c) visited)
		 (when (walk c)
		   (return t)))))))))

;;; ---------------------------- Change History ----------------------------
;;; (Who)   (When) 		     (What)
;;;
