;;;; File: o-face.lsp
;;; Version: %W%
;;; Contains: Interface to TF compiler.
;;; Author: Jeff Dalton and Richard Kirby (rbk)
;;; Created: Thu Jun 18 12:06:13 1992
;;; Updated: Sun Oct 16 01:10:35 1994 by Jeff Dalton
;;; Release Date: %Y%
;;; Copyright: (c) 1992, AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032, rights
;;; in technical data and computer software (foreign) (June 1975)

(in-package :tf-compiler)

;;; Reporting procedures

;;; Syntax errors are reported by calling SYNTAX-ERROR.  COMPILE-TF
;;; arranges for SYNTAX-ERROR to call TF-SYNTAX-ERROR-REPORTER.

(defun tf-syntax-error-reporter (message &rest format-args)
  "The procedure that reports syntax errors detected by the TF compiler."
  (unless *suppress-messages*
    ;; /\/: Allow MON output to appear even if output is supposedly off.
    (let ((*dev-debug-level* dev-minimum-monitor-debug-level))
      (if *include-stack*
	  (tf-message "Error in ~S, line ~D: ~(~?~)"
		      (car *include-stack*) *line-number* message format-args)
	  (tf-message "Error, line ~D: ~(~?~)"
		      *line-number* message format-args)))))

;;; Other errors detected by the TF compiler are reported by calling
;;; TF-ERROR.  Similar procedures for warnings, progress resports, and
;;; general messages are TF-WARNING, TF-PROGRESS and TF-MESSAGE
;;; respectively.

(defun tf-error (message &rest format-args)
  (incf *error-count*)
  (unless *suppress-messages*
    ;; /\/: Allow MON output to appear even if output is supposedly off.
    (let ((*dev-debug-level* dev-minimum-monitor-debug-level))
      (tf-message "Error: ~?" message format-args))))

(defun tf-warning (message &rest format-args)
  (unless *suppress-messages*
    (tf-message "Warning: ~?" message format-args)))

(defun tf-message (message &rest format-args)
  (unless *suppress-messages*
    (if *running-as-tfc*
	(mon-format t "~&~?~%" message format-args)
	(MON:MONITOR :TF
	   (:DETAILED (:NO-MODULE-PREFIX)
	      ;; Print the prefix ourself so we can put ~& in front of it.
	      "~&~A:~?~%" :TF message format-args)))))

;;; TF-PROGRESS is used to begin a new line of progress description.
;;; TF-SUB-PROGRESS can then be used to print more items on the same line
;;; and to print the final newline.

(defun tf-progress (message &rest format-args)
  (unless *suppress-messages*
    (when *progress-reports*
      (if *running-as-tfc*
	  (progn
	    (mon-format t "~&~?" message format-args)
	    (force-output t))
	(MON:MONITOR :TF (:DETAILED (:NO-NEWLINE)
				    "~?" message format-args))))))

(defun tf-sub-progress (message &rest format-args)
  (unless *suppress-messages*
    (when *progress-reports*
      (if *running-as-tfc*
	  (progn
	    (format t "~?" message format-args)
	    (force-output t))
	(MON:MONITOR :TF (:DETAILED (:NO-MODULE-PREFIX :NO-NEWLINE)
				    "~?" message format-args))))))

;;; End

