;;;; File: krsl-drawer.lsp
;;; SCCS Version: %W%
;;; Contains: KRSL output for the O-Plan system
;;; Author: Brian Drabble and Jeff Dalton
;;; Created: June 1993
;;; Updated: Sat Oct 29 04:52:45 1994 by Jeff Dalton
;;; Release Version: %Y%
;;; Copyright: (c) 1992, 1993, AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032
;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

(in-package :plan-world-viewer)

;;; Print in the plan in KRSL format

(defun output-krsl-plan (nodes)
  (let* ((name (ask-user :output-file "File for KRSL Output"))
	 (temp-name (temp-filename name)))
    (unless (string= name "")
      (with-open-file (out temp-name :direction :output)
	(write-krsl-plan-levels nodes out)
        (write-krsl-plan-nodes nodes out))
      (report-action "Wrote ~A." temp-name))))

(defun write-krsl-plan-levels (nodes &optional (out *window*))
  ;;
  ;; 1. for each level pull out the links to levels above and below
  ;; 2. format each of the remaining nodes and send to file
  ;; 3. for each node pull out its successors and send INTERVAL
  ;;    relations
  ;;
  (let ((*print-length* nil)
        (*print-level* nil)
        (*package* *viewer-io-package*)
        (max-level (get-plan-depth nodes)))
    (format out ";;; O-Plan plan output in KRSL format ~%")
    (format out ";;; This is not yet validated but provides an outline ~
                     of the intended output ~%~%")
    (loop for level from 1 to max-level
	  do
      (format out "(:THE PLAN-EXPANSION :NAME PLAN-TO-LEVEL-~D~%" level)
      (format out "   NODES (:VALUES")
      (let ((pnodes (extract-plan-level nodes level)))
	(dolist (n pnodes) 
	  (format out "~%                    (:THE PLAN :NAME ~S)"
		  (nd-node-name n)))
	(format out ")~%")
	(process-link-values pnodes out) 
	(when (< level max-level)
	  (format out "~&   REFINEMENT ~
                             (:THE PLAN EXPANSION :NAME PLAN-TO-LEVEL-~D)"
		  (1+ level)))
	(format out ")~%~%")))))

(defun process-link-values (pnodes &optional (out *window*))
  (let ((*print-length* nil)
        (*print-level* nil)
        (*package* *viewer-io-package*))
    (format out "   GRAPH~%")
    (format out "         (:VALUES")
    (dolist (n pnodes)
      (dolist (succ (nd-begin-suc n))
	(format out "~%                    ~
                     (INTERVAL-BEFORE (:BEGIN ~S) (:~S ~S))"
		(nd-node-name n)
		(drop-of (ne-tag-end succ))
		(ne-tag-node succ)))
      (dolist (succ (nd-end-suc n))
	(format out "~%                    ~
                     (INTERVAL-BEFORE (:END ~S) (:~S ~S))"
		(nd-node-name n)
		(drop-of (ne-tag-end succ))
		(ne-tag-node succ))))
    (format out ")")))
      
	
(defun write-krsl-plan-nodes (nodes &optional (out *window*))
  ;;
  ;; This is a simplified until TOME and GOST are sent by the Planner
  ;;
  (let ((parent-name nil)
	(children nil)
        (*print-length* nil)
        (*print-level* nil)
        (*package* *viewer-io-package*))
    (dolist (n nodes)
      (setq parent-name (get-parent-name (nd-node-name n)))
      (format out "~%(:THE PLAN") 
      (format out "~%      :NAME ~S" (nd-node-name n))
      (format out "~%      :METHOD ~S" (nd-pattern n))
      (format out "~%      :PARAMETERS NIL")
      (format out "~%      :IN-SERVICE-OF NIL")
      (format out "~%      :PARENT ~S" parent-name)
      (format out "~%      :GOALS NIL")
      (format out "~%      :EST ~S" (nd-est n))
      (format out "~%      :LST ~S" (nd-lst n))
      (format out "~%      :EET ~S" (nd-eft n))
      (format out "~%      :LET ~S" (nd-lft n))
      (format out "~%      :MIN ~S" (nd-min-duration n))
      (format out "~%      :MAX ~S" (nd-max-duration n))
      (format out "~%      :PRECONDITIONS NIL")
      (format out "~%      :EXPANSIONS  ")
      ;;
      ;; check if this is a parent and if so output children
      ;;
      (when (parent-node-p n)
	(setq children (get-all-children nodes n))
	(format out "(:VALUES ~%")
	(format out "                   (:THE PLAN-EXPANSION~%")
	(format out "                         :NAME EXP-~S~%" (nd-node-name n))
	(format out "                         NODES (:VALUES ")
	(dolist (n children) 
	  (format out "~%                                       ~
                       (:THE PLAN :NAME ~S)"
		  (nd-node-name n)))
	(format out ")))~%")
	(process-link-values children out))
      (format out ")~%"))))

(defun get-all-children (nodes pnode)
  ;;
  ;; scan the plan to find all children of this parent node
  ;;
  (let ((pnode-name (nd-node-name pnode)))
    (loop for n in nodes
	  when (child-name-p (nd-node-name n) pnode-name)
	  collect n)))

;;; ---------------------------- Change History ----------------------------
;;;
;;; (Who)   (When) 		     (What)
;;;  jd    22 Jun 93	Created a separate file; random cleanup.
;;;
