;;;; File: interface-package.lsp
;;; SCCS Version: %W%
;;; Contains: Package defining the programmatic interface
;;; Author: Jeff Dalton <J.Dalton@ed.ac.uk>
;;; Created: October 1994
;;; Updated: Sun May  7 00:31:18 1995 by Jeff Dalton
;;; Release Version: %Y%
;;; Copyright: (c) 1994, AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032
;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

;;; The oplan-interface package defines the names in the programmatic
;;; interface to O-Plan.  Most of the actual definitions are in the files
;;; subr-mode.lsp and program-interface.lsp.

;;; In a sense, this package defines the exports of the oplan package;
;;; but rather than have the exports in the definition of the oplan
;;; package, we put them here and have the oplan package use this one.
;;; This is because the oplan package doesn't have a neat definition
;;; in one place.

(in-package :oplan)

(defpackage #:oplan-interface

  (:nicknames #:opi)

  (:import-from #:util #:exit-lisp)

  (:export

     ;; Planning and replanning
     #:plan-for
     #:replan

     ;; Plan statistics
     #:request-plan-statistics-list

     ;; Plan views
     #:request-plan-view-list
     #:request-plan-view
     #:request-psgraph

     ;; World views
     #:request-world-view-list
     #:request-world-view

     ;; Time bounds
     #:request-time-bounds

     ;; Component output
     #:redirect-component-output
     #:set-component-output-stream
     #:set-component-debug-level

     ;; Sending and receiving
     #:ask-oplan
     #:send-to-oplan
     #:receive-from-oplan
     #:receive-else-error
     #:receive-case			;/\/ not defined yet

     ;; Timeouts
     #:timeout				;a condition type
     #:timeout-not-handled		;an error condition type
     #:set-timeout
     #:cancel-timeout
     #:with-timeout			;a macro
     #:call-with-timeout		;function equiv to & used by the macro

     ;; Etc
     #:exit-lisp
     #:add-exit-action			;defined in subr-mode.lsp
     )

  )

;;; ---------------------------- Change History ----------------------------
;;; (Who)   (When) 		     (What)
;;;
