;;;; File: inf-arithmetic.lsp
;;; SCCS Version: %W%
;;; Contains: functions for arithmetic with infinity
;;; Author: Jeff Dalton <J.Dalton@ed.ac.uk>
;;; Created: June 1994
;;; Updated: Wed Jun 22 19:40:13 1994 by Jeff Dalton
;;; Release Version: %Y%
;;; Copyright: (c) 1994, AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032
;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

(in-package :util)

(export '(infp &numberp))
(export '(&+ &- &incf &decf))
(export '(&= &< &<= &> &>=))
(export '(&min &max))

;;; Arithmetic with :inf

(defun-inline infp (x)
  (eq x :inf))

(defun-inline &numberp (x)
  (or (numberp x)
      (infp x)))

(defun-inline &+ (i j)
  (if (or (infp i) (infp j))
      :inf
    (+ i j)))

(defun-inline &- (i j)
  (cond ((infp i) :inf)
	((infp j) (error "Can't subtract infinity from ~S." i))
	(t (- i j))))

(define-modify-macro &incf (&optional (delta 1)) &+)
(define-modify-macro &decf (&optional (delta 1)) &-)

(defun-inline &= (i j)
  (cond ((infp i) (infp j))
	((infp j) nil)
	(t (= i j))))

(defun-inline &< (i j)
  (cond ((infp i) nil)
	((infp j) (not (infp i)))
	(t (< i j))))

(defun-inline &<= (i j)
  (cond ((infp i) (infp j))
	((infp j) t)
	(t (<= i j))))

(defun-inline &> (i j)
  (&< j i))

(defun-inline &>= (i j)
  (&<= j i))

(defun-inline &min (i j)
  (if (&< i j) i j))

(defun-inline &max (i j)
  (if (&> i j) i j))

;;; ---------------------------- Change History ----------------------------
;;; (Who)   (When) 		     (What)
;;;
