;;;; File: mcc-defs.lsp
;;; SCCS Version: %W%
;;; Contains: Module Communication Channel defintions.
;;; Author: Richard Kirby and Jeff Dalton
;;; Created: February 1993
;;; Updated: Wed Jun  8 00:26:06 1994 by Jeff Dalton
;;; Release Version: %Y%
;;; Copyright: (c) 1992, 1993 AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032
;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

;;; This file defines the MCC package and the MCC-MESSAGE structure.

(in-package :mcc)

(use-package '(:dev :util :monitors :pseudo-process :components))

;;; /\/: Can't (use-package :IPC) yet.  See support/mcc-client.lsp.

(export '(mcc-register-with-im 
	  mcc-handler
	  mcc-message			;the type name
	  mcc-make-message		;constructor
	  mcc-msg-sender		;accessors
	  mcc-msg-channel		; .
	  mcc-msg-ident			; .
	  mcc-msg-args			; .
	  mcc-msg->list
	  list->mcc-msg
	  mcc-send-to-leftout
	  mcc-leftout-to-planner
	  mcc-leftout-to-ta
	  mcc-send-to-rightout
	  mcc-rightout-to-exec
	  mcc-rightout-to-world
	  ))

;;; /\/: Shouldn't this really be ipc-message?

(defstruct (mcc-message (:conc-name mcc-)
			#+:undef (:type list)
			(:constructor make-mcc-message)
			(:constructor mcc-make-message
			   (msg-channel
			    msg-ident
			    #+akcl &optional	; AKCL barfs without this! /\/
			    &rest msg-args
			    &aux (msg-sender *whoami*))))
  "Defines the form of messages to send around the MCC.
     msg-sender  - Id of the sender of this message, *whoami* by default.
     msg-channel - Symbolic name of the channel this is sent down.
     msg-ident   - What the message is.
     msg-args    - Any arguments for the message."
  msg-sender
  msg-channel
  msg-ident
  msg-args)

(defun mcc-msg->list (msg)
  (list (mcc-msg-sender msg)
	(mcc-msg-channel msg)
	(mcc-msg-ident msg)
	(mcc-msg-args msg)))

(defun list->mcc-msg (msg-list)
  (destructuring-bind (sender channel ident args)
      msg-list
    (make-mcc-message
      :msg-sender sender
      :msg-channel channel
      :msg-ident ident
      :msg-args args)))

(defun mcc-pretty-print-event (stream event)
  (case mon::*monitor-level*
    ((:MINIMUM :ONE-LINER)
     (mon-format stream "~W:~W" (mcc-msg-ident event) (mcc-msg-args event)))
    (otherwise
     (mon-format stream "From ~W: ~W:~W"
		 (mcc-msg-sender event) (mcc-msg-ident event)
		 (mcc-msg-args event)))))

#+:undef
;; mcc-message has :type list so isn't a type.
;; We get an error in Lucid.
(eval-when (load eval)
  (mon-add-user-def-print-fn 'mcc-message #'mcc-pretty-print-event))

;;; ---------------------------- Change History ----------------------------
;;; (Who)   (When) 		     (What)
;;;
