;;;; File: pop-gensym.lsp
;;; SCCS Version: %W%
;;; Contains: A POP11-like GENSYM function, context layered.
;;; Author: Richard Kirby (rbk) and Jeff Dalton
;;; Created: Tue Mar 13 15:18:45 1990
;;; Updated: Mon Apr  3 21:39:14 1995 by Jeff Dalton
;;; Release Version: %Y%
;;; Copyright: (c) 1992, 1993, 1994 AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032
;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

(in-package :util)

; (export '(pop-gensym make-pop-gensymtable clear-pop-gensymtable))

(defvar *pop-gensymtable* 'not-a-pop-gensymtable)

(defun make-pop-gensymtable ()
  (make-hash-table :size 50 :test #'equal
		   :rehash-size 1.5
		   :rehash-threshold .75))

(defun clear-pop-gensymtable ()
  (clrhash *pop-gensymtable*))

(defun pop-gensym (prefix &optional (count nil))
  
  "Acts like POP11's GENSYM function, ie a seperate count is kept for each
   prefix, and starts from 1. CONTEXT-LAYERED.
   Arguments:
     prefix - a string which acts as the prefix.
     count - for resetting the current count for the given prefix.
   Returns:
     An interned symbol of prefix with count appended to the end, and a
     second value which is the count appended.
   Side Effects:
     the current count for the given prefix is updated, or set up if
     this is the first."

  (macrolet ((pg-gethash (key) `(ctxt:ctxt-gethash ,key *pop-gensymtable*)))
    (let ((num (or count (pg-gethash prefix) 1)))
      (setf (pg-gethash prefix) (1+ num))
      (values
        (intern (concatenate 'string prefix (prin1-to-string num)))
	num))))

;;; ---------------------------- Change History ----------------------------
;;; (Who)   (When) 		     (What)
;;;  jd    23 Feb 93	Moved out of the LISP package and combined
;;;			 with the context-layered version.
;;;  jd    26 May 94    Changed to use ctxt-gethash from the contexts
;;;                      package, and rewrote pop-gensym to be simpler.
;;;
