;;;; File: run-lights.lsp
;;; SCCS Version: %W%
;;; Contains: Code for maintaining a window that shows the running processes
;;; Author: Jeff Dalton <J.Dalton@ed.ac.uk>
;;; Created: February 1993
;;; Updated: Wed Jul 20 16:45:31 1994 by Jeff Dalton
;;; Release Version: %Y%
;;; Copyright: (c) 1992, 1993, AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032
;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

;;; Run lights

(in-package :ipc)

(use-package :x)

(export '(run-lights-on run-lights-off *run-lights-geometry*))

(defvar *run-lights-geometry* "51x1")

(defvar *lights-window* nil)

(defun ensure-lights-window (&optional (window-args nil))
  (unless *lights-window*
    (x-open-and-register-io-win :run-lights
      ;; N.B. later args seem to take precedence
      (append
        (list "-sl" "0")		;save no scrolled-off lines
	(or window-args
	    (list "-title" (if (get-parameter :asexec)
			       "Exec Running Processes"
			       "O-Plan Running Processes")
		  "-name" (if (get-parameter :asexec)
			      "Exec Processes"
			      "O-Plan Processes")
		  "-geometry" *run-lights-geometry*
		  "-fn" "fixed"))))
    (setq *lights-window* (x-get-stream :run-lights))))

#+:undef
(defun display-run-lights (p)
  ;; p is the pprocess that's just about to run or else NIL
  (format *lights-window* "~%~{~S ~}~@[~S~]"
	  (mapcar #'pprocess-name
		  (ldiff *pushed-pprocesses* *push-pprocess*))
	  (if p (pprocess-name p) nil)))

(defun display-run-lights (p)
  ;; p is the pprocess that's just about to run or else NIL
  (let ((out *lights-window*))
    (terpri out)
    (do ((pprocs *pushed-pprocesses* (cdr pprocs))
	 (end *push-pprocess*))
	((eq pprocs end))
      (prin1 (pprocess-name (car pprocs)) out)
      (princ " " out))
    (when p
      (prin1 (pprocess-name p) out))
    (force-output out)))		;/\/ do we need this?

(defun run-lights-on (&optional (window-args nil))
  (ensure-lights-window window-args)
  (display-run-lights *pprocess*)
  (advice+ 'run-pprocess 'run-lights
    #'(lambda (next)
	#'(lambda (p)
	    (display-run-lights p)
	    (unwind-protect (funcall next p)
	      (display-run-lights nil)))))
  (advice+ 'call-in-pprocess-env 'run-lights
    #'(lambda (next)
	#'(lambda (p fn)
	    (display-run-lights p)
	    (unwind-protect (funcall next p fn)
	      (display-run-lights nil)))))
  t)

(defun run-lights-off ()
  (mapc #'(lambda (f)
	    (advice- f 'run-lights))
	'(run-pprocess call-in-pprocess-env))
  (format *lights-window* "~%")
  t)

;;; End
